<?php
/*
schreinerwissen. auftragsverwaltung 
Copyright (C) 2011  Carsten Hilbert
c-hilbert@web.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


 /**
  * Class for setting document data. 
  * 
  * For PDF documents, use the class Doc_Pdf in modul/pdf/. 
  * For HTML documents use the class Doc_Html
  * For example invoice, delivery note, order-confirmation etc.
  *
  * Invoice-Structure:
  * 1) positions
  * 2) sub total (zwischensumme)/net(netto)/total1 (rechnungsbetrag1)
  * 3) tax
  * 4) total2 (rechnungsbetrag2) (brutto)
  * 5) partpayment
  * 6) total3 (rechnungsbetrag3)
  *
  * Required: 
  * Function proof_font_size
  * Class Html_Element
  * 
  * @since 2010-12
  */

class Doc extends Dbc 
{
    protected $adr = array(); // Client data
    protected $address; // String with whole address
    protected $address_delivery; // String with delivery address
    protected $adm = array(); // own data
    protected $adr_deliv = array();
    protected $adr_id;
    protected $amount_net; // amount without tax
    protected $amount_net_format; // amount without tax
    protected $amount_gross; // amount with tax
    protected $amount_gross_format; // amount with tax
    protected $amount_net_a;
    protected $amount_net_b;
    protected $amount_sum_total_format;
    protected $amount_output; // 
    protected $arr_company_xy; // Coordinate for company name
    protected $arr_company_data_xy; // Coordinate for company data in head
    protected $arr_logo_xy; // Coordinate for logo
    protected $auf_id_nr; // Order-id
    protected $bank_details;
    protected $date_doc; // document date
    protected $delivery_date;
    protected $delivery_address_exist; // Boolean, if there is a delivery address = true
    protected $delivery_time;
    protected $doc_name;
    protected $font_size;
    protected $font_size_small;
    protected $font_size_big;
    protected $head_company;
    protected $head_document;
    protected $iban_bic;
    protected $let = array(); // Letter data
    protected $ord = array(); // Order data
    protected $order_id; // Order id
    protected $order_tax_rate_a;
    protected $order_tax_rate_b;
    protected $worker = array();
    protected $partpay_gross;
    protected $partpay_net;
    protected $partpay_total_format;
    protected $pay_hint;
    protected $position;
    protected $position_rows = array(); // Contains the records of each position
    protected $postcode_city;
    protected $postcode_city_delivery;
    protected $price_output;
    protected $pos_shipping_output;
    protected $salutation; // Salutation address
    protected $salutation_deliv; // Salutation delivery address
    protected $sender;
    protected $shipping_costs_output;
    protected $small_businessman_output;
    protected $subject_output;
    protected $subject_shipping_output;
    protected $subject_total_part_pay_output;
    protected $subject_pos_part_pay_output;
    protected $tax_a_b_output;
    protected $tax_invoice_included_output;
    protected $tax_shipping_a_b_output;
    protected $tax_number_id;
    protected $tax_rows = array(); // Tax a and tax b
    protected $text_bottom;
    protected $text_field_bottom;
    protected $part_pay_tax_rate_output;
    protected $tax_amount_a; // Will be created in make_tax() in a loop
    protected $tax_amount_b; // Will be created in make_tax() in a loop
    protected $total_format;
    protected $unit_for_output; // 
    protected $width_all;
    protected $word_total_net;
    protected $word_total_gross; // Inscription of line total gross
    protected $word_sum_total; // Inscription of line total gross

    
    /**
     * @var <string> $doc Dokumenttyp:
     * 
     *      offer =    Offer (Kostenvoranschlag),
     *      confi =    Order confirmation (Auftragsbestaetigung),
     *      deliv =    Delivery note (Lieferschein),
     *      invoi =    Invoice (Rechnung),
     *      assum =    Assumption of costs (Kostenbernahmeerklrung),
     *      partp =    Part payment (Abschlag)
     *      lette =    Letter
     * 
     */
    public $doc;
    
    /**
     * Change $doc to column name in table order_item
     * @var type 
     */
    private $doc2 = array(
        'offer' => 'offer',
        'confi' => 'order_conf',
        'deliv' => 'delivery',
        'invoi' => 'invoice',
        );

    /**
     *
     * @var type  Output format p, h, r
     * p = PDF
     * h = HTML
     * r = RETURN Text
     */
    protected $output_format = 'p';

/**
 * Set data from order
 * 
 * @global type $tbl_wor
 * @global type $tbl_adr
 * @global type $tbl_let
 * @global type $arr_salutation
 * @global type $adm
 * 
 * @param int $auf_id
 * @param string $dok_typ
 */
function set_data($auf_id, $dok_typ) 
{
    global $tbl_ord; // necessary!
    global $tbl_wor;
    global $tbl_adr;
    global $tbl_let;
    global $arr_salutation;
    global $adm;

    
    $this->order_id = $auf_id;
    $this->doc = $dok_typ;
    $this->adm = $adm;

    // Proof font_size
    $this->font_size = proof_font_size($this->adm['font_size']);
    $this->font_size_small = $this->font_size * 0.7;
    $this->font_size_big = $this->font_size * 1.2;

    // Operator
    $this->worker = $this->get_row_as_array($tbl_wor, "id = ".$_SESSION['bea_id']);

    // Get data and address id
    // For letters
    if ($this->doc == 'lette') {
        // Get letter data as array
        $this->let = $this->get_row_as_array($tbl_let, "id = ".$auf_id);
        $this->adr_id = $this->let['id_tbl_address'];

    // For orders: show n as array
    }else{
        $this->ord = $this->get_row_as_array($tbl_ord, "id = ".$auf_id);
        $this->adr_id = $this->ord['id_tbl_address'];

        // get address for delivery as array
        $this->adr_deliv = $this->get_row_as_array($tbl_adr, "id = "
                         . $this->ord['id_tbl_address_deliv']);
        /**
         * Nr or id
         */
        if($this->adm['own_doc_number'] == 1) {
            $this->auf_id_nr = $this->ord['number'];
        }else{
            $this->auf_id_nr = $this->order_id;
        }
    }


    // Get address data as array
    $this->adr = $this->get_row_as_array($tbl_adr, "id = ".  $this->adr_id);
    
    // Salutation
    if ($this->adr['id_tbl_salutation'] == 7) {
        $this->salutation = '';
        
    }else{
        $this->salutation = $arr_salutation[$this->adr['id_tbl_salutation']];
    }

    // Salutation delivery
    if (!empty($this->adr_deliv) ) {
        if ($this->adr_deliv['id_tbl_salutation'] == 7) {
            $this->salutation_deliv = '';
        }else{
            $this->salutation_deliv = $arr_salutation[$this->adr_deliv['id_tbl_salutation']];
        }
        $this->delivery_address_exist = true;
        
    }else{
        $this->delivery_address_exist = false;
    }

    // x and y coordinates for company name
    $this->arr_company_data_xy = explode(',', $this->adm['company_data_xy']);
}


/**
 * Set format of output data
 * 
 * @param string $format
 *  p = PDF (default)
 *  h = HTML
 *  r = RETURN (text output without printing)
 */
function set_output_format($format) 
{
    $this->output_format = $format;
}


/**
 * Gets the address id
 *
 * @return integer address_id 
 */
function get_address_id() 
{
    return $this->adr_id;
}


/**
 * Get bank details and aim for paying
 *
 * @global type $la_bank_details
 * @global type $la_pay_hint1
 * @global type $la_invoice_amount_payed
 * @global type $la_bank_account_nr
 * @global type $la_bank_code_number_short
 * @global type $la_days_b
 */
function make_bank_and_aim() 
{
    global $la_bank_details;
    global $la_pay_hint1;
    global $la_invoice_amount_payed;
    global $la_bank_account_nr;
    global $la_bank_code_number_short;
    global $la_days_b;
    global $la_tax_number;
    global $la_tax_id;

    /**
     * Set payed or not
     */
    // if payed
    if ($this->ord['payed'] > 1) {
        $this->pay_hint = $la_invoice_amount_payed . "\n";

    // or not payed
    }else{
        if( !empty($this->ord['payed']) ) {
            // Don't display hint, when time_payment is 0 or null
            if (empty($this->ord['time_payment'])) {
                $this->pay_hint = '';
                
            }else{
                $this->pay_hint = $la_pay_hint1 . $this->ord['time_payment']
                                . ' ' . $la_days_b . ' '.  $this->ord['time_payment_add'];
            }
        }else{
            $this->pay_hint = '';
        }
    }

     /**
      * Set bank account
      */
    if(     $this->adm['bank'] != '' 
        and $this->adm['bank_account_number'] != '' 
        and $this->adm['bank_code'] != ''  
        and $this->adm['bank_account_holder'] != '') 
    {
        $this->bank_details = $la_bank_details . ": ". $this->adm['bank_account_holder']
                            . ", " . $this->adm['bank'] . "\n" . $la_bank_code_number_short . ": " 
                            . $this->adm['bank_code'] . ", " . $la_bank_account_nr . ": "
                            . $this->adm['bank_account_number'] . "\n";
    }

    // Tax number
    if (!empty($this->adm['tax_number']) and !empty($this->adm['tax_id'])) {
        $this->tax_number_id = $la_tax_id . ': ' . $this->adm['tax_id'];
        
    }elseif(!empty($this->adm['tax_number']) and empty($this->adm['tax_id'])) {
        $this->tax_number_id = $la_tax_number . ': ' . $this->adm['tax_number'];
        
    }elseif(empty($this->adm['tax_number']) and !empty($this->adm['tax_id'])) {
        $this->tax_number_id = $la_tax_id . ': ' . $this->adm['tax_id'];
    }

    // Display iban and bic if they are not empty
    if ( !empty($this->adm['iban']) ) {
        $iban = "IBAN: ". $this->adm['iban'];
        $this->iban_bic = $iban;
    }
    
    if ( !empty ($this->adm['bic']) ) {
        $bic =  "BIC: " . $this->adm['bic'];
        if (isset($this->iban_bic)) {
            $this->iban_bic .= ', '. $bic;
        }else{
            $this->iban_bic = $bic;
        }
    }

}


/**
 * To be implemented in child class
 */
function print_bank_and_aim()
{
    
}


/**
 * Makes address or delivery address in a loop
 *
 * @global type $tbl_cou
 * @return type 
 */
function make_address() 
{
    global $tbl_cou;

    // a = invoice address, d = delivery address
    $array = array('a', 'd');
    foreach ($array as $value) {
    
        // If there is no delivery address
        if ( $value == 'd' and $this->delivery_address_exist == false) {
            return false;
        }

        // Switch address or delivery address
        if ($value == 'a') {
            $arr = $this->adr;
            $salut = $this->salutation;

        }elseif($value == 'd') {
            $arr = $this->adr_deliv;
            $salut = $this->salutation_deliv;
        }

        // Company
        if($arr['company'] != '') {
            $address = $arr['company'] . "\n";
        }else{
            $address = '';
        }

        // Address addition
        if ( !empty ($arr['addition'])) {
            $address_addition = $arr['addition'] . "\n";
        }else{
            $address_addition = '';
        }
        
        // Surname forename
        if(empty ($arr['forename']) and empty ($arr['surname'])) {
            $surname_forename = '';
        }else{
            $surname_forename = $salut . ' ' . $arr['forename'] . ' '
                  . $arr['surname'] . "\n";
        }

        // Pre- and surname
        $address  .= $surname_forename;
        $address  .= $address_addition;
        $address  .= $arr['street'] . "\n";

        // Get country
        $country = $this->get_one_field($tbl_cou, $arr['id_tbl_country'], 'country');

        // Postcoce and city
        $postcode_city = $arr['postcode'] . " ". $arr['city'];
        if ($arr['id_tbl_country'] != 'DE') {
            $postcode_city .= "\n" . $country;
        }

        // Switch address or delivery address
        if ($value == 'a') {
            $this->address = $address;
            $this->postcode_city = $postcode_city;
        }elseif($value == 'd') {
            $this->address_delivery = $address;
            $this->postcode_city_delivery = $postcode_city;
        }
    }
}


/**
 * To be implemented in child class
 */
function print_address() 
{
    $text = $this->address . "\n" . $this->postcode_city;
    return $text;
}


/**
 * To be implemented in child class
 */
function print_delivery_address() 
{
    $text = $this->address_delivery . "\n". $this->postcode_city_delivery;
    return $text;
    
}


/**
 * To be implemented in child class
 */
function make_address_kind() 
{
    $this->make_address('d');
    // Switch address or delivery address
    if ($this->doc != 'deliv') {
        $address = $this->address_delivery;
        $postcode_city = $this->postcode_city_delivery;
        $address_kind = $la_delivery_address;
    }else {
        $this->make_address('a');
        $address = $this->address;
        $postcode_city = $this->postcode_city;
        $address_kind = $la_invoice_address;
    }
}


/**
 * Makes company name
 */
function make_company_name() 
{
    // Split value to x and y
    $this->arr_company_xy = explode(',', $this->adm['company_xy']);
    
}

/**
 * To be implemented in child class
 */
function print_company_name() 
{
    
}


/**
 * Makes logo
 */
function make_logo() 
{
    // Split value to x and y
    $this->arr_logo_xy = explode(',', $this->adm['logo_xy']);
    
}

/**
 * To be implemented in child class
 */
function print_logo() 
{
    
}

/**
 *
 */
function make_sender() 
{
    // Next row to string
    $company_name = preg_replace('/\r\n|\r|\n/', ' ', $this->adm['company_name']);

    // Sender
    $this->sender = $company_name . ", " . $this->adm['street'] . ", "
                 . $this->adm['postcode'] . " " . $this->adm['city'];  
}

/**
 * To be implemented in child class
 */
function print_sender() 
{
    
}


/**
 *
 * @global type $la_completed
 * @global type $la_delivered
 * @return type 
 */
function make_delivery_date() 
{
    global $la_completed;
    global $la_delivered;
    
    if ($this->doc == 'invoi' and $this->ord['delivered'] > 1) {
        /**
         * Delivery date
         */
        $date = new DateTime($this->ord['delivered']);

        // if time_format ist empty, no date will be displayed
        $order_delivery_date = $date->format($this->adm['time_format_doc']);
        if (strlen ($order_delivery_date) < 7) {
            $order_delivery_date = $date->format('d.m.Y');
        }
        $text_delivery_date = $this->ord['order_description'] . "\n";
        if (isset($this->ord['delivered_on']) and $this->ord['delivered_on'] == 1) {
            $text_delivery_date .= $la_completed . "/" . $la_delivered 
                                . ": " .  $order_delivery_date;
        }
        $this->delivery_date = $text_delivery_date;
        
        
    }elseif($this->doc == 'invoi' and $this->ord['delivered'] < 1) {
        $date = new DateTime();
        $current_date = date($this->adm['time_format_doc']);
        $text_delivery_date = $this->ord['order_description']."\n";
        if (isset($this->ord['delivered_on']) and $this->ord['delivered_on'] == 1) {
            $text_delivery_date .= $la_completed . "/" . $la_delivered.
            ": " . $current_date;
        }
        $this->delivery_date = $text_delivery_date;

    }else{
        $text_delivery_date = $this->ord['order_description'];
        $this->delivery_date = $text_delivery_date;
    }
    
        if($this->output_format == 'r') {
            return $this->delivery_date;
        }

    
}


/**
 * To be implemented in child class
 */
function print_delivery_date() 
{
    
}


/**
 * Print Positions of the order as a table
 * 
 * @global <type> $tbl_ord_ite
 * @global <type> $tbl_uni
 * @global <type> $la_sum
 * @global <type> $la_invoice_amount
 * @global <type> $la_back
 * @global <type> $la_alternativ
 * @return string
 */
function make_pos() 
{
    global $tbl_ord_ite;
    global $tbl_uni;
    global $la_doc_without_items;
    global $la_invoice_amount;
    global $la_back;
    global $la_alternativ;
    global $la_maybe;

    $text = '';
    $i = 0;
    /**
     * No positions in case of partpayment invoice
     */
    if ($this->doc != 'partp') {
        /**
         *  Display every position
         */
        if ($this->doc == 'be') {
            $this->doc = 'invoi';
        }
        $amount_net = 0;
        $amount_net_a = 0;
        $amount_net_b = 0;
        $qry_auf_pos = "
            SELECT *
            FROM $tbl_ord_ite
            WHERE id_tbl_order = '$this->order_id' 
                AND hide != 1 
                AND {$this->doc2[$this->doc]} = 1
                AND item_typ != 5 
            ORDER BY "
         
        // first part of positon f.i. 2.4 => 2
        . " CAST(SUBSTRING_INDEX(item, '.', 1) AS DECIMAL), "
                
         // second part of positon f.i. 2.4 => 4
        . " CAST(SUBSTRING_INDEX(item, '.', -1) AS DECIMAL) "
                
        . " ASC";
        
        $number_of_result = $this->count_rows($qry_auf_pos);

        if ($number_of_result < 1) {
            $text = $la_doc_without_items;

            // Box if no items
            $info = new Info_Box();
            $condition_info = ($number_of_result < 1);
            $info->set_condition($condition_info);
            $text = $la_doc_without_items;
            $info->show_box($text);

            // Back
            print link_back_with_query();
            exit ('</body></html>');
        }

        $res_auf_pos = $this->query($qry_auf_pos);

        // Start loop
        while($order_pos = $this->fetch_array($res_auf_pos)) {

            // Get total
            $total = $order_pos['sales_price'] * $order_pos['amount'];

            // total sum
            if($order_pos['item_typ'] == 0) {
                // total sum for tax_a and tax_b separately
                if ($order_pos['tax_a_b'] == 1) {
                    $amount_net_a += $total;
                }  elseif ($order_pos['tax_a_b'] == 2) {
                    $amount_net_b += $total;
                }
                $amount_net += $total;
            }

            // Get unit
            $this->unit_for_output =  $this->get_one_field($tbl_uni, $order_pos['id_tbl_unit'], 'unit');

            // Position
            $this->position = $order_pos['item'];

            // Hide price by delivery and text, only if it is no delivery note
            if($this->doc != 'deliv' or $order_pos['item_typ'] == 0) {
                $this->price_output = number_format($order_pos['sales_price'], 2, ",", ".");
            }else{
                $this->price_output = '';
            }

            /**
             *  total alternativ-position in brackets
             */
            // Normal position
            if ($order_pos['item_typ'] == 0) {
                if($this->doc != 'deliv' ) {
                    $total_format = number_format($total, 2, ",", ".");
                }elseif($order_pos['item_typ'] == 1) {
                    $total_format = "(".number_format($total, 2, ",", ".") . ")";
                }else{
                    $total_format = '';
                }

                $this->amount_output = $order_pos['amount'];

                $var = $this->amount_output;
                
                // if value is integer
                if (intval($var) == $var) {
                    $this->amount_output = number_format($this->amount_output, 0);
                    
                // or not integer
                }else{
                    // Two decimal places if unit is not "m3"
                    if ($order_pos['id_tbl_unit'] != 6) {
                        $this->amount_output = number_format($this->amount_output, 2, ',', '.');
                    }
                }

                $this->subject_output = $order_pos['description'];

            // Alternativ position
            }elseif ($order_pos['item_typ'] == 1) {
                $total_format = '';
                $this->subject_output = $la_alternativ . ': ' . $order_pos['description'];

            //  Text position
            }elseif ($order_pos['item_typ'] == 2) {
                $this->position = '';
                $total_format = '';
                $this->price_output = '';
                $this->amount_output = '';
                $this->unit_for_output = '';
                $this->subject_output = $order_pos['description'];
                
            //  Maybe position
            }elseif ($order_pos['item_typ'] == 3) {
                $total_format = '';
                //$this->menge55 = '';
               // $this->einheit55 = '';
                $this->subject_output = $la_maybe . ': ' . $order_pos['description'];
                
            //  Text position with position number
            }elseif ($order_pos['item_typ'] == 4) {
                $total_format = '';
                $this->price_output = '';
                $this->amount_output = '';
                $this->unit_for_output = '';
                $this->subject_output = $order_pos['description'];
            }

             /**
              *  Tax
              */
            // Get tax_a from order. if ,0 hide decimal
            $ust_a = $this->ord['tax_rate_a'];
            $ust_a = explode('.', $ust_a);
            if ($ust_a[1] == 0) {
                $ust_a = $ust_a[0];
            }else{
                $ust_a = $this->ord['tax_rate_a'];
            }


            // Get tax_b from order. if ,0 hide decimal
            $ust_b = $this->ord['tax_rate_b'];
            $ust_b = explode('.', $ust_b);
            if ($ust_b[1] == 0) {
                $ust_b = $ust_b[0];
            }else{
               $ust_b = $this->ord['tax_rate_b'];
            }

            $this->order_tax_rate_a = $ust_a;
            $this->order_tax_rate_b = $ust_b;

            // define tax for each position
             if ($this->ord['tax_on'] == 1) {

                // Normal position
                if( $order_pos['item_typ'] == 0) {
                    switch ($order_pos['tax_a_b']) {
                        case 0:
                            $this->tax_a_b_output = 0;
                            break;
                        case 1:
                           $this->tax_a_b_output = $ust_a;
                            break;
                        case 2:
                            $this->tax_a_b_output = $ust_b;
                            break;
                    }
                    // Add %
                    $this->tax_a_b_output .= '%';
                    

                // Not normal position
                }else{
                    $this->tax_a_b_output = '';
                }

            // If tax is switched off
            }else{
                $this->tax_a_b_output = '';
            }

            // Hide in delivery note
            if ($this->doc == 'deliv') {
                $this->price_output = '';
                $total_format = '';
                $this->tax_a_b_output = '';
            }

            // Write data of each row to array
                $arr_pos = array(
                    $this->position, 
                    $this->amount_output, 
                    $this->unit_for_output, 
                    $this->subject_output,
                    $this->price_output, 
                    $total_format, 
                    $this->tax_a_b_output,  
                        );
            $this->position_rows[$i] = $arr_pos;
            
            // Dont show Pos in text or email
            unset ($arr_pos[0]); 
            
            $text .= implode('  ', $arr_pos) . "\n";
            
            $i++;
        } // end while output postions

        // Set netto betrag
        $this->amount_net = $amount_net;
        $this->amount_net_a = $amount_net_a;
        $this->amount_net_b = $amount_net_b;
        
        return $text;

    /**
     * If part payment
     */
    }else{
        $this->make_pos_part_pay();
    }
}


/**
 * To be implemented in child class 
 */
function print_pos() 
{

}


/**
 * Shipping costs
 *
 * @global  $tbl_ord_ite
 * @global  $la_shipping
 * @global string $la_tax_short
 * @return string 
 */
function make_shipping() 
{
    global $tbl_ord_ite;
    global $la_shipping;
    global $la_tax_short;
    $text = '';
    /**
     * If shipping > null and document is not delivery note
     */
   if ( ($this->ord['shipping_cost'] > 0 
           and $this->doc != 'deliv'
           and $this->doc != 'partp'
           )
           or $this->ord['shipping_cost_always'] == 1) 
   {
       // Get max position
        $condition = "id_tbl_order = " . $_SESSION['auf_id']
                   . " AND hide != 1";
        $pos_max = $this->get_max($tbl_ord_ite, ' CAST(item AS DECIMAL) ', $condition);
        $this->pos_shipping_output = $pos_max + 1;

        // if tax a is bigger
        if ($this->amount_net_a > $this->amount_net_b) {
            //$ust_a_b55 = 'A';
            $this->tax_shipping_a_b_output = $this->order_tax_rate_a;

            $this->amount_net_a += $this->ord['shipping_cost'];

        // if tax b is bigger
        }elseif($this->amount_net_b > $this->amount_net_a) {
            //$ust_a_b55 = 'B';
            $this->tax_shipping_a_b_output = $this->order_tax_rate_b;
            $this->amount_net_b += $this->ord['shipping_cost'];
        }else{
            $this->tax_shipping_a_b_output = 0;
        }
        // Add %
        $this->tax_shipping_a_b_output .= '%';

        // display shipping costs not in delivery note
        if ($this->doc != 'deliv') {
            $this->shipping_costs_output = $this->ord['shipping_cost'];
            $this->shipping_costs_output = number_format($this->shipping_costs_output, 2, ",", ".");
        }else{
            $this->shipping_costs_output = 0;
        }

        $this->amount_net += $this->ord['shipping_cost'];


        // Hide in delivery note
        if ($this->doc == 'deliv') {
            $this->shipping_costs_output = '';
            $this->tax_shipping_a_b_output = '';
        }
        
        // if tax is off don't print tax rate
        if ($this->ord['tax_on'] != 1) {
            $this->tax_shipping_a_b_output = '';
        }

        $this->subject_shipping_output = $la_shipping;
        
        if($this->output_format == 'r') {
            if ($this->doc == 'deliv') {
               $text = $this->subject_shipping_output;
               $text .= "\n";

            }else{
               #$text = $this->pos_shipping_output . '   1  ' . $la_shipping . '   '
               $text = '1 x ' . $la_shipping . '   '
                     . $this->ord['shipping_cost'] . '  ' . $this->adm['currency'];
               $text .= $la_tax_short . $this->tax_shipping_a_b_output;
               $text .= "\n";
            }
        }
   }
   #return $this->tax_shipping_a_b_output;
   return $text;
 }


/**
 * To be implemented in child class
 */
function print_shipping() 
{
    
}
 

/**
 * Returns the included tax
 *
 * @global type $la_following_tax_included
 * @global string $la_tax_short
 * @return type 
 */
function make_tax_invoice_included() 
{
    global $la_following_tax_included;
    global $la_tax_short;
    
    if($this->ord['tax_on'] == 1  and $this->ord['invoice_gross'] == 1 
            and $this->doc != 'deliv')
    {
        $ust_a = $this->amount_net_a / (100 + $this->ord['tax_rate_a']) *
                $this->ord['tax_rate_a'];
        $ust_b = $this->amount_net_b / (100 + $this->ord['tax_rate_b']) *
                $this->ord['tax_rate_b'];

        /**
         * Format tax
         */
        $tax_amount_a_format = number_format($ust_a, 2, ",", ".");
        $tax_amount_b_format = number_format($ust_b, 2, ",", ".");

        $text_description = $la_following_tax_included . ":\n"
                         . $la_tax_short . $this->order_tax_rate_a . "% = "
                         . $tax_amount_a_format .$this->adm['currency']. "\n";
        $text2 = $la_tax_short . $this->order_tax_rate_b . "% = " 
               . $tax_amount_b_format . ' ' . $this->adm['currency'];

        $this->tax_invoice_included_output = $text_description . $text2;

        if($this->output_format == 'r') {
            return $this->tax_invoice_included_output . "\n\n";
        }
    }
}


/**
 * To be implemented in child class
 */
function print_tax_invoice_included() 
{
    
}


/**
 * Makes the document head with data from company Phone etc.
 * 
 * @global type $la_phone
 * @global type $la_date
 * @global type $la_client_no
 * @global type $la_worker
 * @global type $la_your_mark
 * @global type $la_fax
 * @global type $la_mobil
 */
function make_head() 
{
    global $la_phone;
    global $la_date;
    global $la_client_no;
    global $la_worker;
    global $la_your_mark;
    global $la_fax;
    global $la_mobil;
    
    $this->make_doc_name();

    // Not a letter
    if($this->doc != 'lette') {
        $your_mark = $this->ord['your_mark'];

        if($this->adm['own_doc_number'] == 1) {
            $id_doc = $this->ord['number'];
        }else{
            $id_doc = $this->order_id;
        }

    // Letter
    }else{
        $your_mark = $this->let['your_mark'];
    }

    // If customer id = 0 set empty string
    if ($this->adr['id_tbl_client'] == 0) {
        $client_no = '';
    }else{
        $client_no = $this->adr['id_tbl_client'];
    }


    $this->head_company = $this->adm['street']."\n".
        $this->adm['postcode'] ." ". $this->adm['city']. " \n".
        $la_phone.": ". $this->adm['phone']." \n";

        if (!empty($this->adm['mobil'])) {
          $this->head_company .=  $la_mobil.": ".$this->adm['mobil']."\n";
        }
        if (!empty($this->adm['fax'])) {
          $this->head_company .=  $la_fax.": ".$this->adm['fax']."\n";
        }
        if (!empty($this->adm['homepage'])) {
          $this->head_company .= $this->adm['homepage']."\n";
        }
        if (!empty($this->adm['email'])) {
          $this->head_company .= $this->adm['email']."\n";
        }

    $this->head_document = "$la_date: $this->date_doc\n"
                        . "$la_client_no: $client_no\n"
                        . "$la_worker: " . $this->worker['surname'] . "\n"
                        . "$la_your_mark: " . $your_mark;
        
    if ($this->doc != 'lette') {
        $this->doc_name .= ' ' . $id_doc;
        
    }
}

/**
 * Makes the name of the document like INVOICE, OFFER etc.
 * 
 * @global type $la_invoice
 * @global type $la_final_invoice
 * @global type $la_invoice_delivery_note
 * @global type $la_order_conf
 * @global type $la_quotation
 * @global type $la_kostenuebernahme1
 * @global type $la_delivery_note
 * @global type $la_partpay_invoice
 * @global type $la_offer 
 */
function make_doc_name() 
{
    global $la_invoice;
    global $la_final_invoice;
    global $la_invoice_delivery_note;
    global $la_order_conf;
    global $la_quotation;
    global $la_kostenuebernahme1;
    global $la_delivery_note;
    global $la_partpay_invoice;
    global $la_offer;

    $date_alarm = '??-??-??';
    // Invoice bottom
    if ($this->doc == 'invoi') {
        if ($this->ord['part_pay_amount'] == 0) {
            $this->doc_name = $la_invoice;
        }  else {
            $this->doc_name = $la_final_invoice;
        }
        if ($this->ord['date_invoice'] < 1) {
            $this->date_doc = $date_alarm;
        }else{
            $this->date_doc = substr($this->ord['date_invoice'], 0, 10);
        }
        $text_ob = 'invoice_top';
        $this->text_field_bottom = 'invoice_bottom';

    // invoice delivery
    }elseif($this->doc == 'invoi_deliv') {
        $this->doc_name = $la_invoice_delivery_note;
        if ($this->ord['date_invoice'] < 1) {
            $this->date_doc = $date_alarm;
        }else{
            $this->date_doc = substr($this->ord['date_invoice'], 0, 10);
        }
        $text_ob = 'invoice_top';
        $this->text_field_bottom = 'invoice_bottom';
        $this->doc = 'invoi';

    // order_conf bottom
    }elseif($this->doc == 'confi') {
        $text_ob = 'order_conf_top';
        $this->doc_name = $la_order_conf;
        $this->date_doc =  substr($this->ord['date_order_conf'], 0, 10);
        $this->text_field_bottom = 'order_conf_bottom';

    // offer bottom
    }elseif($this->doc == 'offer') {
        $text_ob = 'offer_top';
        $this->doc_name = $la_quotation;
        $this->date_doc =  substr($this->ord['date_offer'], 0, 10);
        $this->text_field_bottom = 'offer_bottom';

    // assumption of costs bottom
    }elseif($this->doc == 'assum') {
        $this->doc_name = $la_kostenuebernahme1;
        $this->date_doc =  substr($this->ord['date_insert'], 0, 10);
        $this->doc = 'invoi';
        $this->text_field_bottom = 'assumption_costs_bottom';

    // lie
    }elseif($this->doc == 'deliv') {
        $this->doc_name = $la_delivery_note;
        $this->text_field_bottom = 'delivery_bottom';
        // for shop: date_insert auf lieferschein auch wenn delivered noch leer ist
        if ($this->ord['delivered'] > 1) {
            $this->date_doc =  substr($this->ord['delivered'], 0, 10);
        }elseif($this->ord['delivered'] < 1) {
            $this->date_doc = date('Y-m-d');
        }

    // Part payment
    }elseif($this->doc == 'partp') {
        $text_ob = 'invoice_top';
        $this->doc_name = $la_partpay_invoice;
        $this->date_doc =  substr($this->ord['date_part_pay'], 0, 10);
        $this->text_field_bottom = 'invoice_bottom';

    // offer top
    }elseif($this->doc == 'offer') {
        $text_ob = 'offer_top';
        $this->doc_name = $la_offer;
        $this->date_doc =  substr($this->ord['date_offer'], 0, 10);
        $this->doc = 'offer';
        $this->text_field_bottom = 'offer_bottom';

    // letter
    }elseif($this->doc == 'lette') {
        $this->doc_name = '';
        $this->date_doc =  substr($this->let['date_letter'], 0, 10);
        $text_ob = '';
        $this->text_field_bottom = '';

    }else{
        $text_ob = '';
        $this->text_field_bottom = '';
        $this->date_doc = '';
        $this->doc_name = '';
    }
    
    // Format date
    if ($this->date_doc != '' and $this->date_doc != $date_alarm) {
        $date = new DateTime($this->date_doc);
        $this->date_doc =  $date->format($this->adm['time_format_doc']);

        // Don't print date with less than 7 numbers
        if (strlen ($this->date_doc) < 7) {
            $this->date_doc = $date->format('d.m.Y');
        }
    }
}


/**
 * To be implemented in child class
 */
function print_head() 
{
    
}


/**
 * To be implemented in child class
 */
function print_legende() 
{
    
}


/**
 * To be implemented in child class
 */
function print_letter() 
{
    
}


/**
 * Makes delivery time
 * 
 * @global type $la_delivery_time 
 */
function make_delivery_time() 
{
    global $la_delivery_time;
    // time_delivery bei offer drucken
    if ($this->ord['time_delivery'] != '' and $this->doc == 'offer') {
        $this->delivery_time = $la_delivery_time . ': '.  $this->ord['time_delivery'];
    }
}


/**
 * To be implemented in child class
 */
function print_delivery_time() 
{
    
}


/**
 * Print small businessman
 *
 * @global string $la_small_business 
 */
function make_small_businessman() 
{
    global $la_small_business;

    if ($this->adm['small_business'] == 1) {
        $this->small_businessman_output = $la_small_business;
    }
}


/**
 * To be implemented in child class
 */
function print_small_businessman() 
{
    
}


/**
 * Printing text on the bottom of the document
 *
 * @global type $tbl_tex
 */
function make_text() 
{
    $str = 'la_' . $this->text_field_bottom . '_content';
    global $tbl_tex;

    $field = $this->text_field_bottom;
    if (!empty($this->text_field_bottom)) 
    {
        if ($this->ord[$field] != 0 )
        {
            // Text default
            if ($this->ord[$field] == 1) 
            {
                $this->text_bottom = $GLOBALS[$str];

            // Freetext
            } else {
                $this->text_bottom = $this->get_one_field($tbl_tex, $this->ord[$this->text_field_bottom], 'text_block');
            }
        }
    }
}


/**
 * To be implemented in child class
 */
function print_text() 
{
    
}


/**
 * Gets the name of sum
 * 
 * @global  $la_invoice_amount
 * @global  $la_sum
 * @return string
 */
 function make_word_total_net() 
{
    global $la_invoice_amount;
    global $la_sum;

     /**
      *  Word sum or invoice amount
      */
    // with tax
    if($this->ord['tax_on'] == 1 ) {
        if ($this->ord['invoice_gross'] == 1) {
            if ($this->ord['part_pay_amount'] > 0) {
                $word_total = $la_sum;
            }else{
                $word_total = $la_invoice_amount . ' ' . $this->adm['currency'];
            }
            
        }else{
            $word_total = $la_sum;
        }

    // without tax
    }else{
        // with part payment
        if ($this->ord['part_pay_amount'] > 0) {
            if ($this->doc == 'partp') {
                $word_total = $la_invoice_amount . ' ' . $this->adm['currency'];
            }else{
                $word_total = $la_sum;
            }

        // without part payment
        }else{
            $word_total = $la_invoice_amount . ' ' . $this->adm['currency'];
        }
    }
    $this->word_total_net = $word_total;
    return $word_total;
}


/**
 * Row with net amount
 *
 * @return string 
 */
function make_total_net() {

    $this->make_word_total_net();
    
    // format amount net
    $this->amount_net_format = number_format($this->amount_net, 2, ",", ".");

    // Text
    if($this->output_format == 'r') {
        $text = $this->word_total_net . ' ' . $this->amount_net_format . "\n";
        return $text;
    }
}


/**
 * To be implemented in child class
 */
function print_total_net() 
{

}


/**
 * Rows with tax
 *
 * @global string $la_tax_short
 * @return string 
 */
function make_tax() 
{
    global $la_tax_short;

    $text = '';
    
    // Necessary for printing two documents
    $this->tax_rows = array();
    
    /**
     * TAX_A_B
     */
    if($this->ord['tax_on'] == 1  and $this->ord['invoice_gross'] != 1) {
        
        // Loop for A and loop for B
        $tax = array('a', 'b');
        foreach ($tax as $v) {
            
            // variable like pattern ust_betrag_a, tax_b
            $amount_net = 'amount_net_' . $v;
            $auf_ust_satz = 'order_tax_rate_' . $v;
            $tax_inscription = 'tax_inscription_' . $v;
            $tax_amount = 'tax_amount_' . $v;

            if ( $this->$amount_net > 0 ) {
                $tax_amount_a_b = ($this->$amount_net) / 100 * $this->ord['tax_rate_'.$v];
                $this->$tax_amount = $tax_amount_a_b;
                $tax_inscription_a_b = $la_tax_short . ' ' . $this->$auf_ust_satz . '%';
                $tax_amount_format_a_b =  number_format($tax_amount_a_b, 2, ",", ".");

                // Fill array
                $arr_pos = array( $tax_inscription_a_b, '', $tax_amount_format_a_b, '');
                $this->tax_rows[]= $arr_pos;

                // Make string for return text
                if($this->output_format == 'r') {
                    $text .= $tax_inscription_a_b . ' ' . $tax_amount_format_a_b . "\n";
                }
            } // end if 
        } // end foreach
    } // end if
    return $text;
}


/**
 * To be implemented in child class
 */
function print_tax() 
{

}


/**
 * Get total amount
 * 
 * @return string 
 */
function make_total_gross() 
{
    
    if($this->ord['tax_on'] == 1  and $this->ord['invoice_gross'] != 1) {
        
        $this->make_word_total_gross();

        /**
         *  invoice amount gross
         */
        $this->amount_gross = $this->amount_net;
        // Add tax A
        if (isset($this->tax_amount_a)) {
            $this->amount_gross += $this->tax_amount_a;
        }
        // Add tax B
        if (isset($this->tax_amount_b)) {
            $this->amount_gross += $this->tax_amount_b;
        }

        $invoice_amount_gross_format = number_format($this->amount_gross, 2, ",", ".");
        
        $this->amount_gross_format = $invoice_amount_gross_format;
        
        if($this->output_format == 'r') {
             $text = $this->word_total_gross . ' ' . $invoice_amount_gross_format . "\n";
             return $text;
        }
    } // end without invoice_gross
}

/**
 *
 * @global type $la_amount_gross
 * @global  $la_invoice_amount
 * @return string 
 */
function make_word_total_gross() 
{
    global $la_amount_gross;
    global $la_invoice_amount;
    
    // Partpayment
    if ($this->ord['part_pay_amount'] > 0) {
        if (        $this->doc == 'partp' 
                or  $this->doc == 'confi'
                or  $this->doc == 'offer' 
                or ($this->doc == 'invoi' and $this->ord['part_pay_payed'] < 1)) 
        {
            $word_total_gross = $la_invoice_amount . ' ' . $this->adm['currency'];
        }else{
           $word_total_gross = $la_amount_gross;
        }

    // Without partpayment
    }else{
       $word_total_gross = $la_invoice_amount . ' ' . $this->adm['currency'];
    }

    $this->word_total_gross = $word_total_gross;
    return $word_total_gross;
}


/**
 * To be implemented in child class
 */
function print_total_gross() 
{
    
}


/**
 * Gets the row part payment
 *
 * @global type $order
 * @global type $la_partpay_for_nr
 * @return string 
 */
function make_pos_part_pay() 
{
    global $order;
    global $la_partpay_for_nr;

    $this->subject_pos_part_pay_output = $this->ord['part_pay_description'];
    $price_output = $this->ord['part_pay_amount'];
    $this->total_format = number_format($price_output, 2, ",", ".");
    $this->amount_output = 1;
    $this->unit_for_output = '';
    $this->amount_net = $price_output;
    
     if ($this->ord['tax_on'] == 1) {
        // tax rate for partpayment
        if ($this->ord['part_pay_tax_a_b'] == 0) {
            $this->part_pay_tax_rate_output = '--';
            
        }  elseif ($this->ord['part_pay_tax_a_b'] == 1) {
            $this->part_pay_tax_rate_output = $this->ord['tax_rate_a'] . '%';
            $this->amount_net_a = $price_output;
            
        }  elseif ($this->ord['part_pay_tax_a_b'] == 2) {
            $this->part_pay_tax_rate_output = $this->ord['tax_rate_b'] . '%';
            $this->amount_net_b = $price_output;
        }
        
        // Tax rate for row with tax
        $this->order_tax_rate_a = $this->ord['tax_rate_a'];
        $this->order_tax_rate_b = $this->ord['tax_rate_b'];
        
     }else{
         $this->part_pay_tax_rate_output = '';
     }
    
    if($this->output_format == 'r') {
    $text .= $this->amount_output . ' ' . $this->unit_for_output
          . ' ' . $this->subject_pos_part_pay_output . ' '
          . $this->total_format . ' ' . $this->total_format . ' ' . "\n";
    return $text;
    }
}


/**
 * To be implemented in child class
 */
function print_pos_part_pay() 
{

}


/**
 * Total of part payment
 *
 * @global type $la_partpay_payed_at
 * @global  $la_invoice_amount
 * @return boolean 
 */
function make_part_pay() 
{
    global $la_partpay_payed_at;
    global $la_invoice_amount;

    if ($this->ord['part_pay_amount'] > 0 and $this->ord['part_pay_payed'] > 1) {
        if (    $this->doc != 'partp' 
            and $this->doc != 'confi'
            and $this->doc != 'offer'
        ) {
            
            // With tax
            if ( $this->ord['tax_on'] == 1) {
                $partpay_gross_net = ($this->ord['part_pay_amount'] +
                        ($this->ord['part_pay_amount'] / 100 * $this->ord['tax_rate_a'])) * -1;
                $this->partpay_gross = $partpay_gross_net;
               
            // Without tax
            }else{
                $partpay_gross_net = $this->ord['part_pay_amount']  * -1;
                $this->partpay_net = $partpay_gross_net;
            }
            
            $part_pay_total_format = number_format($partpay_gross_net, 2, ",", ".");
            
            $subject_part_pay = $la_partpay_payed_at . ' '.  $this->ord['part_pay_payed'];
            $subject_part_pay .= ': ' . $this->ord['part_pay_description'];
            $this->subject_total_part_pay_output = $subject_part_pay;
            $this->partpay_total_format = $part_pay_total_format;            
        } // end if

        return true;
        
    }else{
        return false;
    }
}


/**
 * To be implemented in child class
 */
function print_part_pay() 
{
    
}


/**
 * Makes total of part payment
 * 
 * @global type $la_partpay
 * @global  $la_invoice_amount 
 */
function make_sum_total() 
{
    global $la_partpay;
    global $la_invoice_amount;
    
    // Amount with part payment
    if (    $this->doc != 'partp' 
        and $this->doc != 'confi'
        and $this->doc != 'offer'
        and $this->ord['part_pay_amount'] > 0 
        and $this->ord['part_pay_payed'] > 1) 
    {
        // With tax
        if ( $this->ord['tax_on'] == 1) {
            if ($this->ord['invoice_gross'] == 1) {
                $invoice_amount_minus_partpay = $this->amount_net 
                        + $this->partpay_gross;
            }else{
                $invoice_amount_minus_partpay = $this->amount_gross 
                        + $this->partpay_gross;
            }

        // Without tax
        }else{
            $invoice_amount_minus_partpay = $this->amount_net + $this->partpay_net;
        }

        $invoice_amount_gross_minus_partpay_format = number_format($invoice_amount_minus_partpay, 
                2, ",", ".");

        $this->word_sum_total = $la_invoice_amount . ' ' . $this->adm['currency'];
        $this->amount_sum_total_format = $invoice_amount_gross_minus_partpay_format;

            $text = $this->word_sum_total . $this->amount_sum_total_format 
                  . "\n";
            return $text;

    }else{
        return false;
    }        
}


/**
 * To be implemented in child class
 */
function print_sum_total() 
{
    
}


/**
 * To be implemented in child class
 */
function print_line() 
{
    
}


/**
 * Gets the font-size
 */
function get_font_size() 
{
    return $this->font_size;
}


/**
 * Gets the font-size
 */
function get_font_size_small() 
{
    return $this->font_size_small;
}


/**
 * Gets the font-size
 */
function get_font_size_big() 
{
    return $this->font_size_big;
}


/**
 * set width of horizontal line
 * @param <type> $param
 */
function set_width_all($param) 
{
    $this->width_all = $param;
}


 /**
  * Compose the complete document
  * 
  * @return string complete invoice as a string
  */
function print_totals() 
{
    /**
     *  1) Get positions
     */
    $text = $this->make_pos();
    $this->print_pos();
    if ($this->doc == 'partp') {
        $this->print_pos_part_pay();
    }
    
    
   if (((       $this->ord['shipping_cost'] > 0  and $this->doc != 'deliv' )
           or   $this->ord['shipping_cost_always'] == 1 ) 
           and  $this->doc != 'partp' ) 
   {
        $text .= $this->make_shipping();
        $this->print_shipping();
   }
   $this->print_line();

    /**
     * Exit when dok is delivery
     */
    if ($this->doc == 'deliv') {
        return false;
    }

    /**
     *  2) Get net (netto)/zwischensumme/rechnungsbetrag1
     */
    $text .= $this->make_total_net();
    $this->print_total_net();
    
    /**
     *  3) Get tax
     */
    $text .= $this->make_tax();
    $this->print_tax();
    
    /**
     *  4) Get total (rechnungsbetrag1)
     */
    if ($this->ord['tax_on'] == 1 and $this->ord['invoice_gross'] != 1) {
        $this->print_total_gross();
        $text .= $this->make_total_gross();
        
    }

    /**
     *  5) Get total (partpayment)
     */
    $text .= $this->print_part_pay();
    
    /**
     *  5) Get total (sum_total)
     */
    $this->print_sum_total();
    $text .= $this->make_sum_total();
   
    if ($this->output_format == 'r') {
            return $text . "\n";
    }
}


/**
 * Update amount in tbl_order
 * 
 * @global string $tbl_ord
 */
function __destruct() 
{
    global $tbl_ord;

    if ($this->doc != 'lette') {
        if ($this->ord['tax_on'] == 1 and $this->ord['invoice_gross'] != 1) {
            $invoice_amount_net_or_gross = $this->amount_gross;
        }else{
            $invoice_amount_net_or_gross = $this->amount_net;
        }
        if ($this->doc == 'invoi') {
            $aendern = "
            UPDATE $tbl_ord 
            SET invoice_amount = '$invoice_amount_net_or_gross',
                tax_a = '$this->tax_amount_a',
                tax_b = '$this->tax_amount_b'
            WHERE id = " . $_SESSION['auf_id'];
            $this->do_sql($aendern);
        }
    }
}


} // end class
?>